@@ -49,17 +49,19 @@ struct _GFloppy {
 	GFloppyType type;
 	GFloppySize size;
 	GFloppyFormattingMode formatting_mode;
+
 	gint nblocks;      /* total number of 512 byte blocks */
 	gint nbadblocks;       /* number of bad blocks found during scan
 				  (only available in the parent process) */
 	GList *badblocks_list; /* list of the bad blocks found during scan
 				  (only available in the child process) */
+
 	gboolean mkdosfs_backend; /* If true, the mkdosfs FAT formatting
 				     will be used */
-	gchar *volume_name;    /* the label to be assigned to the disk */
-	gchar *device;         /* ie. /dev/fd0  can also be /dev/fd0H1440 */
-	gchar *extended_device;/* ie. /dev/fd0H1440 */
-	gchar *mdevice;        /* ie. a: or b: */
+	char *volume_name;     /* the label to be assigned to the disk */
+
+	char *device;          /* ie. /dev/fd0, /dev/fd0H1440 or /dev/floppy/0 (devfs) */
+	char *mdevice;         /* ie. a: or b: */
 
         /* child process which does actual work */
 	gint pid;
@@ -75,10 +77,10 @@ struct _GFloppy {
 	gint message[2];
 
 	/* Our commands */
-	gchar *badblocks_cmd;
-	gchar *mformat_cmd;
-	gchar *mke2fs_cmd;
-	gchar *mkdosfs_cmd;
+	char *badblocks_cmd;
+	char *mformat_cmd;
+	char *mke2fs_cmd;
+	char *mkdosfs_cmd;
 };
 /* This should do the format of the floppy. */
 void format_floppy (GFloppy *floppy);
@@ -92,7 +94,7 @@ typedef enum {
 	GFLOPPY_DEVICE_OK
 } GFloppyStatus;
 
-GFloppyStatus test_floppy_device (gchar *device);
+GFloppyStatus test_floppy_device (char *device);
 gint	      floppy_block_size  (GFloppySize size);
 
 #endif
